/*
 *  filter.c - Filtering mechanism
 *
 *  Copyright (C) 1998 Hugo Haas
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <ctype.h>
#include <fnmatch.h>

#include "defines.h"
#include "filter.h"

struct filter_entry *filter = NULL;

/*
 * check_range
 *
 * Check if a number is in a particular range
 */

int check_range(const __u16 number, const long min, const long max, const short protocol) {
  __u16 n;

  if ((min == NOT_DEFINED) && (max == NOT_DEFINED))
    return TRUE;

  if (protocol == TCP)
    n = (number << 8) + (number >> 8);
  else
    n = number;

  if (min == NOT_DEFINED) {
    if (n <= max)
      return TRUE;
  }
  else {
    if (max == NOT_DEFINED) {
      if (n >= min)
	return TRUE;
    }
    else {
      if ((n <= max) && (n >= min))
	return TRUE;
    }
  }

  return FALSE;
}

/*
 * check_address
 *
 * Check if an address is described by address/mask
 */

int check_address(const __u32 host, const __u32 address, const __u32 mask) {

  if (address == 0)
    return TRUE;

  if (((address ^ host) & mask) == 0)
    return TRUE;

  return FALSE;
}

/*
 * check_name
 *
 * Checks if a hostname matches a pattern
 */

int check_name(const char *name, const char *pattern) {
  char *string;
  const char *c;
  char *d;

  if (!(pattern && *pattern))
    return TRUE;

  if (!(name && *name))
    return FALSE;

  string = (char *) malloc(strlen(name));
  c = name;
  d = string;
  while (*c != ' ') {
    *d = tolower(*c);
    c++;
    d++;
  }
  *d = '\0';
  if (!fnmatch(pattern, string, FNM_NOESCAPE)) {
    free(string);
    return TRUE;
  }
  free(string);
  return FALSE;
}

/*
 * do_log
 *
 * Returns TRUE if the packet needs to be logged, FALSE otherwise
 */

int do_log(const __u32 address, const char *hostname, const __u16 type, const short protocol) {
  struct filter_entry *p;

  for(p = filter; p != NULL; p = p-> next) {

    if (!((p->protocol == ALL) || (p->protocol == protocol)))
      continue;

    if (check_range(type, p->range_min, p->range_max, protocol) &&
	check_address(address, p->address, p->mask) &&
	check_name(hostname, p->hostname))
      return p->log;
  }

  return TRUE;
}

/*
 * $Log: filter.c,v $
 * Revision 1.4  1998/11/29 21:51:12  hugo
 * Removed an error in check_range
 *
 * Revision 1.3  1998/11/29 20:48:32  hugo
 * New filtering mechanism (completely rewritten)
 *
 * Revision 1.2  1998/10/29 23:06:59  hugo
 * Began numerical filter implementation
 *
 * Revision 1.1  1998/10/25 23:19:11  hugo
 * Initial revision
 *
 */
