/*
 *  icmp.c - functions logging ICMP messages
 *
 *  Copyright (C) 1998 Hugo Haas
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <netinet/ip.h>
#include <netinet/ip_icmp.h>
#include <unistd.h>
#include <pthread.h>
#include <stdlib.h>

#include "defines.h"
#include "icmp.h"
#include "netutils.h"
#include "log.h"
#include "filter.h"
#include "configuration.h"

extern struct logger log;

extern char *configuration_file;

/*
 * Structure of an ICMP packet
 */

struct icmppacket {
  struct iphdr ip;
  struct icmphdr icmp;
};

/*
 * icmp_type_lookup
 *
 * Gives the type of an ICMP message
 */

void icmp_type_lookup(char *string, __u8 type) {

  switch(type) {
  case ICMP_ECHOREPLY:
    strcpy(string,"echo reply");
    return;
  case ICMP_DEST_UNREACH:
    strcpy(string,"destination unreachable");
    return;
  case ICMP_SOURCE_QUENCH:
    strcpy(string,"source quench");
    return;
  case ICMP_REDIRECT:
    strcpy(string,"redirect");
    return;
  case ICMP_ECHO:
    strcpy(string,"echo request");
    return;
  case ICMP_ROUTER_ADVERT:
    strcpy(string,"router advertisement");
    return;
  case ICMP_ROUTER_SOLICITATION:
    strcpy(string,"router solicitation");
    return;
  case ICMP_TIME_EXCEEDED:
    strcpy(string,"time exceeded");
    return;
  case ICMP_PARAMETERPROB:
    strcpy(string,"parameter problem");
    return;
  case ICMP_TIMESTAMP:
    strcpy(string,"timestamp request");
    return;
  case ICMP_TIMESTAMPREPLY:
    strcpy(string,"timestamp reply");
    return;
  case ICMP_INFO_REQUEST:
    strcpy(string,"information request");
    return;
  case ICMP_INFO_REPLY:
    strcpy(string,"information reply");
    return;
  case ICMP_ADDRESS:
    strcpy(string,"address mask request");
    return;
  case ICMP_ADDRESSREPLY:
    strcpy(string,"address mask reply");
    return;
  default:
    strcpy(string,"unknown");
  }
}

/*
 * log_icmppacket & threaded_log_icmppacket
 *
 * Log an ICMP message
 */

#define TYPE_LENGTH 50

void *threaded_log_icmppacket(void *pkt) {
  char remote_host[HOST_LENGTH];
  char icmp_type[TYPE_LENGTH];

  /* Host filter */
  host_lookup(remote_host, ((struct icmppacket *) pkt)->ip.saddr);
  if (do_log(((struct icmppacket *) pkt)->ip.saddr, remote_host, ((struct icmppacket *) pkt)->icmp.type, ICMP) == TRUE) {
    icmp_type_lookup(icmp_type, ((struct icmppacket *) pkt)->icmp.type);
    log.f(LOGLEVEL, "ICMP message type %s from %s", icmp_type, remote_host);
  }

  free(pkt);
  return NULL;
}

void log_icmppacket(struct icmppacket pkt) {
  pthread_attr_t attr_t;
  pthread_t t;
  __u8 *image;

  image = (__u8 *) malloc(sizeof(struct icmppacket));
  memcpy(image, &pkt, sizeof(struct icmppacket));
  pthread_attr_init(&attr_t);
  pthread_attr_setdetachstate(&attr_t, PTHREAD_CREATE_DETACHED);
  pthread_create(&t, &attr_t, threaded_log_icmppacket, image);
  pthread_attr_destroy(&attr_t);
}

/*
 * log_icmp
 *
 * Main thread logging ICMP messages
 *
 */

void *log_icmp(void) {
  int s;
  struct icmppacket pkt;

  s = socket(AF_INET, SOCK_RAW, ICMP);
  if (s <= 0)
    exit(1);

  for(;;) {
    read(s, (struct icmppacket *) &pkt, sizeof(struct icmppacket));

    if (pkt.ip.ihl != 5) {
      log_suspicious((struct iphdr *) &pkt);
      continue;
    }

    log_icmppacket(pkt);    
  }
  /*@NOTREACHED@*/ 
  return NULL;
}

/*
 * $Log: icmp.c,v $
 * Revision 1.4  1998/11/29 21:51:12  hugo
 * Removed 2 typos
 *
 * Revision 1.3  1998/11/29 20:48:32  hugo
 * Modified to use the new filtering mechanism
 *
 * Revision 1.2  1998/10/25 23:19:11  hugo
 * Added host filtering
 *
 * Revision 1.1  1998/10/25 15:05:00  hugo
 * Initial revision
 *
 */
